﻿using System;
using System.Data;
using System.Configuration;
using System.Web;

using System.Data.SqlClient;

namespace LZ2007.Function.Log
{
    /// <summary>
    /// 数据库日志类.
    /// </summary>
    public class DataBaseLog : ILog
    {
        #region 私有变量申明
        private string _LogConnStr;
        private string _LogObjectSource;

        private bool _isDebug;
        private bool _isInfo;
        private bool _isError;
        private bool _isWarn;

        private SqlConnection _sqlConn;

        public bool IsDebug
        {
            get { return _isDebug; }
            set { _isDebug = value; }
        }
        public bool IsInfo
        {
            get { return _isInfo; }
            set { _isInfo = value; }
        }
        public bool IsError
        {
            get { return _isError; }
            set { _isError = value; }
        }
        public bool IsWarn
        {
            get { return _isWarn; }
            set { _isWarn = value; }
        }
        #endregion

        public DataBaseLog(Type objType)
        {
            _isDebug = Convert.ToBoolean(ConfigurationManager.AppSettings["eLogDebug"]);
            _isInfo = Convert.ToBoolean(ConfigurationManager.AppSettings["eLogInfo"]);
            _isError = Convert.ToBoolean(ConfigurationManager.AppSettings["eLogError"]);
            _isWarn = Convert.ToBoolean(ConfigurationManager.AppSettings["eLogWarn"]);
            _LogObjectSource = objType.FullName;
            //_LogType = Convert.ToInt32(ConfigurationManager.AppSettings["eLogType"]);
            _LogConnStr = ConfigurationManager.AppSettings["eLogConnStr"];
            _sqlConn = new SqlConnection(_LogConnStr);
            Init();
        }

        private void Init()
        {
            //检查是否有该表
            string strTest = "select count(name)as a1 from sysobjects where id = object_id(N'[LOGSYSTEM]') and OBJECTPROPERTY(id, N'IsUserTable') = 1";

            //建立表
            string strSQL = "CREATE TABLE [dbo].[LOGSYSTEM] (" +
                                "[lId] [int] IDENTITY (1, 1) NOT NULL ," +
                                "[lMessage] [nvarchar] (1000) COLLATE Chinese_PRC_CI_AS NULL ," +
                                "[lLevel] [int] NULL ," +
                                "[lSource] [nvarchar] (100) COLLATE Chinese_PRC_CI_AS NULL ," +
                                "[lException] [nvarchar] (100) COLLATE Chinese_PRC_CI_AS NULL ," +
                                "[lType] [int] NULL ," +
                                "[lDate] [datetime] NULL ," +
                                "[lADDIT1] [nvarchar] (100) COLLATE Chinese_PRC_CI_AS NULL ," +
                                "[lADDIT2] [nvarchar] (100) COLLATE Chinese_PRC_CI_AS NULL ," +
                                "[lADDIT3] [nvarchar] (100) COLLATE Chinese_PRC_CI_AS NULL " +
                            ") ON [PRIMARY]";

            _sqlConn.Open();
            SqlCommand sqlcomm = new SqlCommand(strTest, _sqlConn);
            int i = (int)sqlcomm.ExecuteScalar(); ;
            if (i == 0)
            {
                sqlcomm = new SqlCommand(strSQL, _sqlConn);


                sqlcomm.ExecuteNonQuery();
            }
            sqlcomm.Dispose();
        }

        private int insertLog(string message, int level, string source, string exception, int type)
        {

            string strSQL = "INSERT INTO LOGSYSTEM(lMessage,lLevel,lSource,lException,lType,lDate) " +
                "VALUES (@lMessage,@lLevel,@lSource,@lException,@lType,@lDate)";
            SqlCommand sqlcomm = new SqlCommand(strSQL, _sqlConn);
            if (_sqlConn.State == ConnectionState.Closed)
            {
                _sqlConn.Open();
            }
            sqlcomm.Parameters.Add("@lMessage", SqlDbType.NVarChar, 1000);
            sqlcomm.Parameters["@lMessage"].Direction = ParameterDirection.Input;
            sqlcomm.Parameters["@lMessage"].Value = message;

            sqlcomm.Parameters.Add("@lLevel", SqlDbType.Int);
            sqlcomm.Parameters["@lLevel"].Direction = ParameterDirection.Input;
            sqlcomm.Parameters["@lLevel"].Value = level;

            sqlcomm.Parameters.Add("@lSource", SqlDbType.NVarChar, 100);
            sqlcomm.Parameters["@lSource"].Direction = ParameterDirection.Input;
            sqlcomm.Parameters["@lSource"].Value = source;

            sqlcomm.Parameters.Add("@lException", SqlDbType.NVarChar, 100);
            sqlcomm.Parameters["@lException"].Direction = ParameterDirection.Input;
            sqlcomm.Parameters["@lException"].Value = exception;

            sqlcomm.Parameters.Add("@lType", SqlDbType.Int);
            sqlcomm.Parameters["@lType"].Direction = ParameterDirection.Input;
            sqlcomm.Parameters["@lType"].Value = type;

            sqlcomm.Parameters.Add("@lDate", SqlDbType.DateTime);
            sqlcomm.Parameters["@lDate"].Direction = ParameterDirection.Input;
            sqlcomm.Parameters["@lDate"].Value = DateTime.Now;
            return sqlcomm.ExecuteNonQuery();
        }

        public void Close()
        {
            this.Info("Log End", 5);
            _sqlConn.Close();
        }

        #region 信息[Info]
        public void Info(string message, int level)
        {
            if (_isInfo)
            {
                insertLog(message, level, _LogObjectSource, "", 0);
            }
        }

        public void Info(string message)
        {
            if (_isInfo)
            {
                insertLog(message, 0, _LogObjectSource, "", 0);
            }
        }
        #endregion

        #region 警告[Warn]
        public void Warn(string message)
        {
            if (_isWarn)
            {
                insertLog(message, 0, _LogObjectSource, "", 1);
            }
        }

        public void Warn(string message, int leave)
        {
            if (_isWarn)
            {
                insertLog(message, leave, _LogObjectSource, "", 1);
            }
        }
        #endregion

        #region 调试[Debug]
        public void Debug(string message, int leave)
        {
            if (_isDebug)
            {
                insertLog(message, leave, _LogObjectSource, "", 2);
            }
        }

        public void Debug(string message)
        {
            if (_isDebug)
            {
                insertLog(message, 0, _LogObjectSource, "", 2);
            }
        }
        #endregion

        #region 错误[Error]
        public void Error(string message, Exception e, int leave)
        {
            if (_isError)
            {
                if (e != null)
                {
                    insertLog(message, leave, _LogObjectSource, e.Message, 3);
                }
                else
                {
                    insertLog(message, leave, _LogObjectSource, "", 3);
                }
            }
        }

        public void Error(string message, Exception e)
        {
            if (_isError)
            {
                if (e != null)
                {
                    insertLog(message, 0, _LogObjectSource, e.Message, 3);
                }
                else
                {
                    insertLog(message, 0, _LogObjectSource, "", 3);
                }
            }
        }
        #endregion

        #region 致命错误[Fatal]
        public void Fatal(string message, Exception e, int leave)
        {
            if (e != null)
            {
                insertLog(message, leave, _LogObjectSource, e.Message, 4);
            }
            else
            {
                insertLog(message, leave, _LogObjectSource, "", 4);
            }
        }

        public void Fatal(string message, Exception e)
        {
            if (e != null)
            {
                insertLog(message, 0, _LogObjectSource, e.Message, 4);
            }
            else
            {
                insertLog(message, 0, _LogObjectSource, "", 4);
            }
        }
        #endregion

    }
}