﻿using System;
using System.Data;
using System.Configuration;
using System.Web;

using System.IO;

namespace LZ2007.Function.Log
{
    /// <summary>
    /// FileLog 的摘要说明
    /// </summary>
    /// 
    public class FileLog : ILog
    {
        #region 私有变量申明
        private string _LogFile;
        private string _LogMsgFormat;
        private string _LogObjectSource;

        private StreamWriter sw;
        private bool _isDebug;
        private bool _isInfo;
        private bool _isError;
        private bool _isWarn;

        public bool IsDebug
        {
            get { return _isDebug; }
            set { _isDebug = value; }
        }
        public bool IsInfo
        {
            get { return _isInfo; }
            set { _isInfo = value; }
        }
        public bool IsError
        {
            get { return _isError; }
            set { _isError = value; }
        }
        public bool IsWarn
        {
            get { return _isWarn; }
            set { _isWarn = value; }
        }
        #endregion

        public FileLog(Type objType)
        {
            _isDebug = Convert.ToBoolean(ConfigurationManager.AppSettings["eLogDebug"]);
            _isInfo = Convert.ToBoolean(ConfigurationManager.AppSettings["eLogInfo"]);
            _isError = Convert.ToBoolean(ConfigurationManager.AppSettings["eLogError"]);
            _isWarn = Convert.ToBoolean(ConfigurationManager.AppSettings["eLogWarn"]);
            _LogObjectSource = objType.FullName;
            _LogFile = ConfigurationManager.AppSettings["eLogFile"];
            _LogMsgFormat = ConfigurationManager.AppSettings["eLogMsgFormat"];
            Init();
        }

        public void Close()
        {
            this.Info("Log End", 5);
            sw.Close();
        }

        #region 私有函数实现
        private void Init()
        {
            string path = HttpContext.Current.Server.MapPath("~/") + _LogFile;
            sw = File.AppendText(path);
        }

        private void insertLog(string message, int level, string source, string exception, int type)
        {
            string sType = string.Empty;
            string msg = _LogMsgFormat;
            msg = msg.Replace("{$message}", message);
            msg = msg.Replace("{$level}", level.ToString());
            msg = msg.Replace("{$source}", source);
            msg = msg.Replace("{$exception}", exception);

            switch (type)
            {
                case 0:
                    {
                        sType = "Info";
                        break;
                    }
                case 1:
                    {
                        sType = "Warn";
                        break;
                    }
                case 2:
                    {
                        sType = "Debug";
                        break;
                    }
                case 3:
                    {
                        sType = "Error";
                        break;
                    }
                case 4:
                    {
                        sType = "Fatal";
                        break;
                    }
            }
            msg = msg.Replace("{$type}", sType);
            msg = msg.Replace("{$time}", DateTime.Now.ToString());
            sw.WriteLine(msg);
            sw.Flush();
        }
        #endregion

        #region 信息[Info]
        public void Info(string message, int level)
        {
            if (_isInfo)
            {
                insertLog(message, level, _LogObjectSource, "", 0);
            }
        }

        public void Info(string message)
        {
            if (_isInfo)
            {
                insertLog(message, 0, _LogObjectSource, "", 0);
            }
        }
        #endregion

        #region 警告[Warn]
        public void Warn(string message)
        {
            if (_isWarn)
            {
                insertLog(message, 0, _LogObjectSource, "", 1);
            }
        }

        public void Warn(string message, int leave)
        {
            if (_isWarn)
            {
                insertLog(message, leave, _LogObjectSource, "", 1);
            }
        }
        #endregion

        #region 调试[Debug]
        public void Debug(string message, int leave)
        {
            if (_isDebug)
            {
                insertLog(message, leave, _LogObjectSource, "", 2);
            }
        }

        public void Debug(string message)
        {
            if (_isDebug)
            {
                insertLog(message, 0, _LogObjectSource, "", 2);
            }
        }
        #endregion

        #region 错误[Error]
        public void Error(string message, Exception e, int leave)
        {
            if (_isError)
            {
                if (e != null)
                {
                    insertLog(message, leave, _LogObjectSource, e.Message, 3);
                }
                else
                {
                    insertLog(message, leave, _LogObjectSource, "", 3);
                }
            }
        }

        public void Error(string message, Exception e)
        {
            if (_isError)
            {
                if (e != null)
                {
                    insertLog(message, 0, _LogObjectSource, e.Message, 3);
                }
                else
                {
                    insertLog(message, 0, _LogObjectSource, "", 3);
                }
            }
        }
        #endregion

        #region 致命错误[Fatal]
        public void Fatal(string message, Exception e, int leave)
        {
            if (e != null)
            {
                insertLog(message, leave, _LogObjectSource, e.Message, 4);
            }
            else
            {
                insertLog(message, leave, _LogObjectSource, "", 4);
            }
        }

        public void Fatal(string message, Exception e)
        {
            if (e != null)
            {
                insertLog(message, 0, _LogObjectSource, e.Message, 4);
            }
            else
            {
                insertLog(message, 0, _LogObjectSource, "", 4);
            }
        }
        #endregion
    }
}